/**
 * Jsonix is a JavaScript library which allows you to convert between XML
 * and JavaScript object structures.
 *
 * Copyright (C) 2010 Aleksei Valikov, Highsource.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.hisrc.jsonix.compiler;

import org.apache.commons.lang.Validate;
import org.hisrc.jscm.codemodel.expression.JSMemberExpression;
import org.hisrc.jscm.codemodel.expression.JSMemberExpression.MemberNew;
import org.jvnet.jaxb2_commons.xml.bind.model.MBuiltinLeafInfo;
import org.jvnet.jaxb2_commons.xml.bind.model.MClassInfo;
import org.jvnet.jaxb2_commons.xml.bind.model.MEnumLeafInfo;
import org.jvnet.jaxb2_commons.xml.bind.model.MList;
import org.jvnet.jaxb2_commons.xml.bind.model.MPackagedTypeInfo;
import org.jvnet.jaxb2_commons.xml.bind.model.MTypeInfoVisitor;
import org.jvnet.jaxb2_commons.xml.bind.model.MWildcardTypeInfo;

final class CreateTypeInfoDeclarationVisitor<T, C extends T> implements
		MTypeInfoVisitor<T, C, JSMemberExpression> {

	private final JsonixCode code;

	private final JsonixCompiler<T, C> jsonixCompiler;

	CreateTypeInfoDeclarationVisitor(JsonixCompiler<T, C> jsonixCompiler,
			JsonixCode code) {
		Validate.notNull(jsonixCompiler);
		Validate.notNull(code);
		this.code = code;
		this.jsonixCompiler = jsonixCompiler;
	}

	private JSMemberExpression createTypeInfoDeclaration(
			MPackagedTypeInfo<T, C> info, JSMemberExpression typeInfoClass) {
		final JsonixModule module = this.jsonixCompiler.getModule(info);

		final JSMemberExpression typeInfoDeclaration = module.space.p(info
				.getLocalName());

		final MemberNew typeInfoExpression = typeInfoClass.instantiate().args(

				this.code.codeModel.object().append(
						"name",
						this.code.codeModel.string(module.spaceName + "."
								+ info.getLocalName())));

		module.declarations.expression(typeInfoDeclaration.assign(

		typeInfoExpression));

		return typeInfoDeclaration;

	}

	public JSMemberExpression visitEnumLeafInfo(MEnumLeafInfo<T, C> info) {
		// TODO Enums not supported yet, see JSNX-???
		// return null;
		// return createTypeInfoDeclaration(info,
		// this.code.$JsonixModelEnumLeafInfo);
		return this.jsonixCompiler.getTypeInfoDeclaration(info
				.getBaseTypeInfo());
	}

	public JSMemberExpression visitClassInfo(MClassInfo<T, C> info) {
		return createTypeInfoDeclaration(info, this.code.$JsonixModelClassInfo);
	}

	public JSMemberExpression visitList(MList<T, C> info) {
		return this.code.$JsonixSchemaXSDList.instantiate().args(
				this.jsonixCompiler.getTypeInfoDeclaration(info
						.getItemTypeInfo()));
	}

	public JSMemberExpression visitBuiltinLeafInfo(MBuiltinLeafInfo<T, C> info) {
		return null;
	}

	public JSMemberExpression visitWildcardTypeInfo(MWildcardTypeInfo<T, C> info) {
		return null;
	}
}